const makeRequestFactory = require('..');
const fetch = require('node-fetch');

const mocks = {
  'get /api/message': { message: 'hello' },
  'post /api/model/:id': (req, res) => {
    res.send({ id: req.params.id });
  }
};


it('simple', async () => {
  const makeRequest = makeRequestFactory({ prefix: 'https://httpbin.org' }, fetch);
  const response = await makeRequest('post', '/anything/:id')({
    params: { id: 32 },
    query: { pageSize: 10 },
    body: { k: 'v' }
  });
  expect(response.url).toEqual('https://httpbin.org/anything/32?pageSize=10');
  expect(response.method).toEqual('POST');
  expect(response.json).toEqual({ k: 'v' });
});

it('mocks', async () => {
  const makeRequest = makeRequestFactory({ mocks });
  const getMessage = await makeRequest('get', '/api/message')();
  expect(getMessage).toEqual({ message: 'hello' });
  const getModel = await makeRequest('post', '/api/model/:id')({ params: { id: 32 }});
  expect(getModel).toEqual({ id: "32" });
});

it('preFetch', async () => {
  const preFetch = async options => {
    options.body = { k: 'v' };
  }
  const makeRequest = makeRequestFactory({ prefix: 'https://httpbin.org', preFetch }, fetch);
  const response = await makeRequest('post', '/anything/:id')({
    params: { id: 32 },
  });
  expect(response.json).toEqual({ k: 'v' });
});

it('postFetch', async () => {
  const postFetch = async response => {
    response.modified = true;
    return response;
  }
  const makeRequest = makeRequestFactory({ prefix: 'https://httpbin.org', postFetch }, fetch);
  const response = await makeRequest('post', '/anything/:id')({
    params: { id: 32 },
    body: { k: 'v' },
  });
  expect(response.modified).toEqual(true);
});

it('postFetch handle error', async () => {
  const postFetch = async responseOrError => {
    if (responseOrError instanceof Error) {
      return { err: responseOrError };
    }
    return responseOrError;
  }
  const makeRequest = makeRequestFactory({ prefix: 'https://httpbin.org', postFetch }, fetch);
  const response = await makeRequest('get', '/status/400')();
  expect(response).toHaveProperty('err');
});